var irc     = require('irc'),
    http    = require('http'),
    url     = require('url'),
    nconf   = require('nconf'),
    fs      = require('fs') ;

var channels = [];
nconf.file({ file: 'config.js' });

var client = new irc.Client(nconf.get('irc_server'), nconf.get('irc_nick'), {
    userName: 'gitlabbot',
    realName: 'IRC Bot for Gitlab webhooks',
    localAddress: 'webhooks.paivola.fi',
    autoRejoin: true
});

function display_404(url, req, res) {
    res.writeHead(404, {'Content-Type': 'text/html'});
    res.write("<h1>404 Not Found</h1>");
    res.end("The page you were looking for: "+url+" can not be found");
}

function repoNameFromUrl(repoUrl) {
    if (repoUrl.startsWith("http")) {
        var path = url.parse(repoUrl).pathname;
        return path.split('/').slice(-2).join('/').replace(".git", "");
    } else {
        return repoUrl.split(":").pop().replace(".git", "");
    }
}

function handleEnd(req, res) {
    var url_parts = url.parse(req.url),
        queryData = url.parse(req.url, true).query;

    if (typeof req == "undefined" || typeof req.content == "undefined" || req.content == "") { 
        display_404(url_parts.pathname, req, res);
        return;
    }
    var data = JSON.parse(req.content);
    if (typeof data == "undefined") {
        display_404(url_parts.pathname, req, res);
        return;
    }
    console.log(data);
    if (channels.indexOf(queryData.channel) == -1) {
        client.join('#' + queryData.channel );
        channels.push(queryData.channel);
    }
    var reponame = typeof data.repository != "undefined" ? repoNameFromUrl(data.repository.url) : data.object_attributes.target.name;
    switch(data.object_kind) {
        case 'push':
            var branch = data.ref.split("/").pop();
            var multiline = queryData.multiline || false;
            client.notice('#' + queryData.channel, "["+ reponame +"#"+branch+"] "  + data.commits.length + " new commits by '" + data.user_name + "'");
            var commit_messages = data.commits.map( function(commit) {
                if (multiline)
                    client.notice('#' + queryData.channel, "- '" + commit.message.replace(/(\r\n|\n|\r)/gm," ") + "'" );
                else {
                    client.notice('#' + queryData.channel, "- '" + commit.message.split(/\r|\n/)[0] + "'");
                }
            });
            break;
        case 'tag_push':
            client.notice('#' + queryData.channel, "["+ reponame +"#"+branch+"] "  + data.commits.length + " new commits by '" + data.user_name + "' with tag");
            var commit_messages = data.commits.map( function(commit) {
                client.notice('#' + queryData.channel, "- '" + commit.message.replace(/(\r\n|\n|\r)/gm," ") + "'" );
            });
            break;
        case 'issue':
            handleIssue(data, queryData, reponame);
            break;
        case 'note':
            break;
        case 'merge_request':
            client.notice('#' + queryData.channel, "["+ reponame +"] new merge request by '" + data.user.name + "'");
            break;
        default:
            break;
    }
    res.writeHead(202, {}) ;
    res.write('ok') ;
    res.end();
}

function handleIssue(data, queryData, reponame) {
    switch(data.object_attributes.action) {
        case 'open':
            client.notice('#' + queryData.channel, "["+ reponame +"] new issue '" + data.object_attributes.title + "' by '" + data.user.name + "'");
            break;
        case 'update':
            client.notice('#' + queryData.channel, "["+ reponame +"] issue '" + data.object_attributes.title + "' edited by '" + data.user.name + "'");
            break;
        case 'close':
            client.notice('#' + queryData.channel, "["+ reponame +"] issue '" + data.object_attributes.title + "' closed by '" + data.user.name + "'");
            break;
        default:
            break;
    }
}

http.createServer(function (req, res) {
    console.log("Pyyntö vastaanotettu: " + req.url);
    req.setEncoding("utf8");
    req.content = '';

    var url_parts = url.parse(req.url);

    switch(url_parts.pathname) {
        case '/commit':
            req.addListener("data", function(chunk) {
            	req.content += chunk;
            });
            req.addListener("end", function(){ handleEnd(req, res); });
            break;
        default:
            display_404(url_parts.pathname, req, res);
    }
    return;


}).listen(nconf.get('http_port')) ;
console.log("Running on port " + nconf.get('http_port'));
